@@ -24,8 +24,6 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <readline/readline.h>
-#include <readline/history.h>
 #include <stdlib.h>
 #include <netdb.h>
 #include <arpa/inet.h>
@@ -541,15 +539,15 @@ int do_os_scan(agent *a, int ip, void *np, int flags)
 {
 	DEBUG(printf("%s(): osscan for %s\n", __FUNCTION__, ip2str(ip)) );
 	
-	eh->len = htonl(sizeof(event_hdr) + sizeof(os_scan_e));
+	eh->len = htonl(sizeof(event_hdr) + sizeof(os_scan_e) + strlen(options_os_scan_ports));
 	eh->hlen = htons(sizeof(event_hdr));
 	eh->type = htons(EVENT_OS_SCAN);
 	eh->flags = flags;
 	ee->os_scan_e.ip = htonl(ip);
 	ee->os_scan_e.np = np;
 	ee->os_scan_e.options = htonl(options_os_scan_flags);	
-	strncpy(ee->os_scan_e.ports, options_os_scan_ports, sizeof(ee->os_scan_e.ports));
-
+	strcpy(END_OF_OPTION(&(ee->os_scan_e)), options_os_scan_ports);
+	
 	if (event_send(a, eh) < 0) 
 		clog(LOG_WARNING, "Unable to send OS-scan event\n");
 	return(0);
@@ -561,28 +559,31 @@ int handle_os_scan_reply(event_hdr *h, event *e, agent *a)
 	os_scan_option *opt;
 	os_scan_option_os *os;
 	os_scan_option_port *port;
+	os_scan_option_uptime *os_uptime;
 	os_stats	*os_data;
 	os_port_entry	*os_port;
 	int i,j = ntohl(e->os_scan_r.num_options);
 	page_object *po;
-	int ip = ntohl(e->os_scan_r.ip);
+	int ip = e->os_scan_r.ip;
 	net_page *np = e->os_scan_r.np;
 	
 	DEBUG( printf("%s()\n", __FUNCTION__) );
+	DEBUG( printf("%s(): %d options\n", __FUNCTION__, j) );
 	po = page_object_get_by_ip( np, ip);
 	
 	if(po == NULL) // we could not find the page object so dont do anything to it
+	{
+		DEBUG( printf("could not find host for ip=%s\n", inet_ntoa(*((struct in_addr *)&ip))) );
 		return(0);
-
+	}
 	page_object_free_os_data(po);	
 		
-	if( NULL == (os_data = malloc(sizeof(os_stats))) )
+	if( NULL == (os_data = malloc(sizeof(*os_data))) )
 	{
 		printf("erg no memory\n");
 		exit(1);
 	}
-	
-	os_data->ports = NULL;
+	memset(os_data, 0, sizeof(*os_data));
 	
 	for(i = 0; i < j; i++)
 	{
@@ -591,35 +592,53 @@ int handle_os_scan_reply(event_hdr *h, event *e, agent *a)
 		{
 			case OS_SCAN_OPTION_OS:
 				os = (void *)opt;
-				os_data->os = makestring((char *)&os->string);
-				
-				ptr += ntohl(opt->length);
+				os_data->os = makestring(END_OF_OPTION(os));
+				DEBUG( printf("OS_SCAN_OPTION_OS: %s\n", os_data->os) );
+				break;
+			
+			case OS_SCAN_OPTION_UPTIME:
+				os_uptime = (void *)opt;
+				os_data->uptime_seconds = ntohl(os_uptime->seconds);
+				if(strlen(END_OF_OPTION(os_uptime)) != 0)
+					os_data->uptime_last_boot = makestring(END_OF_OPTION(os_uptime));
+				DEBUG( printf("OS_SCAN_OPTION_OS: %d %s\n", os_data->uptime_seconds, os_data->uptime_last_boot) );
 				break;
 			
 			case OS_SCAN_OPTION_PORT:
 				port = (void *)opt;
 				os_port = malloc(sizeof(os_port_entry));
-				os_port->port = ntohs(port->port_number);
-				os_port->protocol = port->protocol;
-				os_port->name = NULL;
+				os_port->port = ntohl(port->port_number);
+				os_port->protocol = ntohl(port->protocol);
 				os_port->version = NULL;
-
-				add_service(os_port->port, os_port->protocol, port->name);
+				os_port->proto = ntohl(port->proto);
+				os_port->state = ntohl(port->state);
+				os_port->rpcnum = ntohl(port->rpcnum);
+				os_port->rpclowver = ntohl(port->rpclowver);
+				os_port->rpchighver = ntohl(port->rpchighver);
+				os_port->name = strdup(END_OF_OPTION(port));
+				os_port->owner = strdup(END_OF_OPTION(port) + strlen(os_port->name) + 1);
+				add_service(os_port->port, os_port->protocol, os_port->name);
 				
 				if(options_probe_ports)
 					do_probe(a, ip, os_port->port, probe_timeout_ms, np);
 				
 				os_port->next = os_data->ports;
 				os_data->ports = os_port;
-				
-				ptr += ntohl(opt->length);
+				DEBUG( printf("OS_SCAN_OPTION_PORT: proto=%d port=%d %s\n", os_port->protocol, 
+				                                                            os_port->port, 
+				                                                            os_port->name) );
+				DEBUG( printf("OS_SCAN_OPTION_PORT: rpcnum=%d ver=%d.%d owner=%s\n", os_port->rpcnum, 
+				                                                                     os_port->rpchighver, 
+				                                                                     os_port->rpclowver, 
+				                                                                     os_port->owner) );
 				break;
 					
 			default:
+				DEBUG( printf("unknown option\n") );
 				clog(LOG_ERROR,"\nWhat the h@#$ is this? handle_os_scan_reply\n");
-				ptr += ntohl(opt->length);
 				break;
 		}
+		ptr += ntohl(opt->length);
 	}
 	po->os_data = os_data;
 
@@ -689,7 +708,7 @@ int handle_map_icmp_reply(event_hdr *h, event *e, agent *a)
 		clog(LOG_NOTICE, "i got a map icmp reply for %s\n", inet_ntoa(*(struct in_addr *)&dest) );		
 		clog(LOG_NOTICE, "  %s\tto %s\n", inet_ntoa( *(struct in_addr *)&ip1)
 	                                , inet_ntoa( *(struct in_addr *)&ip2) );
-		clog(LOG_NOTICE, "  np = %n", np);
+		clog(LOG_NOTICE, "  np = %p\n", np);
 	)
 
 	if(ip1 != ip2)
@@ -758,7 +777,7 @@ int handle_probe_reply(event_hdr *h, event *e, agent *a)
 	int ip = ntohl(e->probe_r.ip);
 	u16 port = ntohs(e->probe_r.port);
 	net_page *np = e->probe_r.np;
-	char *version = strdup( ((char *)&e->probe_r.np + sizeof(e->probe_r.np)) );
+	char *version = ((char *)&e->probe_r.np + sizeof(e->probe_r.np));
 	page_object *po;
 	os_port_entry *os_port, *prev, *new_os;
 	int found = FALSE;
@@ -768,7 +787,7 @@ int handle_probe_reply(event_hdr *h, event *e, agent *a)
 		clog(LOG_NOTICE, "i got a probe reply for %s\n", inet_ntoa(*(struct in_addr *)&ip) );		
 		clog(LOG_NOTICE, "port %d\n", port);
 		clog(LOG_NOTICE, "np = %p\n", np);
-		clog(LOG_NOTICE, "versoin = '%s'\n", version);
+		clog(LOG_NOTICE, "version = '%s'\n", version);
 	)
 
 	po = page_object_get_by_ip(np, ip);
@@ -791,8 +810,8 @@ int handle_probe_reply(event_hdr *h, event *e, agent *a)
 		else
 		{
 			os_stats *os_data = malloc(sizeof(*os_data));
+			memset(os_data, 0, sizeof(*os_data));
 			os_data->os = strdup("unknown");
-			os_data->ports = NULL;
 			
 			po->os_data = os_data;
 			DEBUG(printf("no os_data\n"));
@@ -812,6 +831,7 @@ int handle_probe_reply(event_hdr *h, event *e, agent *a)
 			if(service)
 			{
 				new_os = malloc(sizeof(*os_port));
+				memset(new_os, 0, sizeof(*new_os));
 				new_os->port = port;
 				new_os->protocol = 6; // tcp
 				new_os->version = strdup(version);
@@ -843,16 +863,113 @@ int handle_probe_reply(event_hdr *h, event *e, agent *a)
 	return(0);
 }
 
+int send_auth_request(agent *a, char *username, char *password)
+{
+	printf("authenticating %s %s\n", username, password);
+
+	eh->len = htonl(sizeof(event_hdr) + sizeof(auth_request_e));
+	eh->hlen = htons(sizeof(event_hdr));
+	eh->type = htons(EVENT_AUTH_REQUEST);
+	eh->flags = 0;
+	strcpy(ee->auth_request_e.username, username);
+	strcpy(ee->auth_request_e.password, password);
+	
+	if (event_send(a, eh) < 0) 
+		clog(LOG_WARNING, "Unable to send login event\n");
+	
+	return(0);
+}
+
+int handle_auth_reply(event_hdr *h, event *e, agent *a)
+{
+	int authenticated = ntohl(e->auth_request_r.authenticated);
+
+	if(authenticated)
+		a->flags |= AGENT_FLAGS_LOGGED_IN;
+	else
+	{
+		GtkWidget *username;
+		GtkWidget *password;
+		GtkWidget *dialog;
+		GtkWidget *hbox;
+		GtkWidget *label;
+		
+		a->flags &= ~(AGENT_FLAGS_LOGGED_IN);
+		
+		dialog = gnome_dialog_new("Please log in",
+		                          GNOME_STOCK_BUTTON_OK,
+		                          GNOME_STOCK_BUTTON_CANCEL,
+		                          NULL);
+		gnome_dialog_close_hides(GNOME_DIALOG(dialog), FALSE);
+		gtk_signal_connect(GTK_OBJECT(dialog), "destroy", gtk_object_destroy, NULL);		
+		gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(main_window->window));
+		
+		hbox = gtk_hbox_new(FALSE, 5);
+
+		label = gtk_label_new("Username:");
+		username = gnome_entry_new("login_username");
+		
+		gtk_box_pack_start( GTK_BOX(hbox), label, FALSE, FALSE, 5);
+		gtk_box_pack_start( GTK_BOX(hbox), username, FALSE, FALSE, 5);
+		gtk_box_pack_start( GTK_BOX( GNOME_DIALOG(dialog)->vbox ), hbox, TRUE, TRUE, 0);
+
+		gtk_widget_show(label);	
+		gtk_widget_show(username);
+		gtk_widget_show(hbox);
+
+		hbox = gtk_hbox_new(FALSE, 5);
+
+		label = gtk_label_new("Password:");
+		password = gtk_entry_new();
+		gtk_entry_set_visibility(GTK_ENTRY(password), FALSE);
+		
+		gtk_box_pack_start( GTK_BOX(hbox), label, FALSE, FALSE, 5);
+		gtk_box_pack_start( GTK_BOX(hbox), password, FALSE, FALSE, 5);
+		gtk_box_pack_start( GTK_BOX( GNOME_DIALOG(dialog)->vbox ), hbox, TRUE, TRUE, 0);
+
+		gtk_widget_show(label);	
+		gtk_widget_show(password);
+		gtk_widget_show(hbox);
+
+		gtk_widget_grab_focus(GTK_COMBO(username)->entry);
+
+		gtk_signal_connect(GTK_OBJECT(GTK_COMBO(username)->entry), "activate", GTK_SIGNAL_FUNC(gtk_widget_focus_me), password);
+		gtk_signal_connect(GTK_OBJECT(GTK_ENTRY(password)),  
+		                   "activate", 
+		                   GTK_SIGNAL_FUNC(click_ok_on_gnome_dialog), 
+		                   dialog);
+		switch( gnome_dialog_run(GNOME_DIALOG(dialog)) )
+		{
+			case 0:
+				send_auth_request(a, 
+				                  gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(username)))),
+				                  gtk_entry_get_text(GTK_ENTRY(password)) );
+				gnome_dialog_close(GNOME_DIALOG(dialog));
+				break;
+			case 1:	
+				gnome_dialog_close(GNOME_DIALOG(dialog));
+				break;
+			case 2:	
+				gnome_dialog_close(GNOME_DIALOG(dialog));
+				break;
+			default:
+				break;
+		}
+		
+	}
+	return(0);
+}
 
 /* *****************************       HANDLERS        ************************* */
 
 struct handler gui_handlers[] = {
-	{ REPLY_ERROR,		handle_error,		"Error Reply" },
-	{ REPLY_DISCOVER_IPV4,	handle_discover_reply,	"Discover Reply Handler" },
-	{ REPLY_OS_SCAN,	handle_os_scan_reply,	"OS Scan Reply Handler" },
-	{ REPLY_DNS_QUERY,	handle_dns_query_reply,	"DNS Query Reply Handler" },
-	{ REPLY_MAP_ICMP,	handle_map_icmp_reply,	"MAP ICMP Reply Handler" },
-	{ REPLY_PROBE,		handle_probe_reply,		"PORT PROBE Handler" },
+	{ REPLY_ERROR,         handle_error,            "Error Reply" },
+	{ REPLY_DISCOVER_IPV4, handle_discover_reply,   "Discover Reply Handler" },
+	{ REPLY_OS_SCAN,       handle_os_scan_reply,    "OS Scan Reply Handler" },
+	{ REPLY_DNS_QUERY,     handle_dns_query_reply,  "DNS Query Reply Handler" },
+	{ REPLY_MAP_ICMP,      handle_map_icmp_reply,   "MAP ICMP Reply Handler" },
+	{ REPLY_PROBE,         handle_probe_reply,      "PORT PROBE Handler" },
+	{ REPLY_AUTH_REQUEST,  handle_auth_reply,       "authentication Handler" },
 	{ -1 },
 };
 
