@@ -3,14 +3,23 @@
  * 
  * Brent Priddy <toopriddy@mailcity.com>
  *
- * Copyright(C) 1999, Adtran, Inc.
+ * Copyright(C) 1999 Brent Priddy <toopriddy@mailcity.com>
  * 
- * Distributed under the terms of the GNU General Public License (GPL) Version
- *
- * Header file for the service callback stuff (spawning a program that has 
- * something to do with the port/protocol that is running on a host)
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  */
+#ifndef GUI_SERVICE_H
+#define GUI_SERVICE_H
+
+#include "cheops-gui.h"
+
 typedef struct _service_list_t {
 	char                   *name;
 	int                     port;
@@ -21,7 +30,7 @@ typedef struct _service_list_t {
 
 extern service_list_t *service_list;
 
-extern int service_callback(int ip, int port, int protocol); 
+extern int service_callback(page_object *po, int port, int protocol); 
 extern service_list_t *service_list_add(char *name, int port, int protocol, char *string);
 extern service_list_t *service_list_change(service_list_t *service, char *name, int port, int protocol, char *string);
 extern void service_list_default(void);
@@ -33,6 +42,7 @@ extern char *service_list_get(int port, int protocol);
 extern service_list_t *services_make_dialog(service_list_t *service, char *name, int port, int protocol, char *str, GtkWidget *parent);
 extern os_pixmap_list_t *os_pixmaps_make_dialog(os_pixmap_list_t *os_pixmap, char *pix, char *str, GtkWidget *parent);
 
+int run_command_callback(page_object *po, int port, int protocol, char *string);
 
-
+#endif /* GUI_SERVICE_H */
 
